@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2003-2013 GraphicsMagick Group
+% Copyright (C) 2003-2016 GraphicsMagick Group
 % Copyright (C) 2002 ImageMagick Studio
 % Copyright 1991-1999 E. I. du Pont de Nemours and Company
 %
@@ -44,6 +44,7 @@
 #include "magick/attribute.h"
 #include "magick/blob.h"
 #include "magick/colormap.h"
+#include "magick/enum_strings.h"
 #include "magick/log.h"
 #include "magick/magick.h"
 #include "magick/monitor.h"
@@ -1098,6 +1099,12 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
             image->units=PixelsPerInchResolution;
           if (jpeg_info.density_unit == 2)
             image->units=PixelsPerCentimeterResolution;
+          if (image->logging)
+            (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                                  "Image resolution set to %gx%g %s",
+                                  image->x_resolution,
+                                  image->y_resolution,
+                                  ResolutionTypeToString(image->units));
         }
     }
   /*
@@ -1271,18 +1278,42 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
   image->depth=Min(jpeg_info.data_precision,QuantumDepth);
   if (jpeg_info.out_color_space == JCS_GRAYSCALE)
     if (!AllocateImageColormap(image,1 << image->depth))
-      ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,image);
+      {
+        jpeg_destroy_decompress(&jpeg_info);
+        ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,image);
+      }
   if (image_info->ping)
     {
       jpeg_destroy_decompress(&jpeg_info);
       CloseBlob(image);
       return(image);
     }
+  if (CheckImagePixelLimits(image, exception) != MagickPass)
+    {
+      jpeg_destroy_decompress(&jpeg_info);
+      ThrowReaderException(ResourceLimitError,ImagePixelLimitExceeded,image);
+    }
+
+  /*
+    Verify that we support the number of output components.
+  */
+  if ((jpeg_info.output_components != 1) &&
+      (jpeg_info.output_components != 3) &&
+      (jpeg_info.output_components != 4))
+    {
+      jpeg_destroy_decompress(&jpeg_info);
+      ThrowReaderException(CoderError,ImageTypeNotSupported,image);
+    }
+
   jpeg_pixels=MagickAllocateArray(JSAMPLE *,
 				  jpeg_info.output_components,
-				  image->columns*sizeof(JSAMPLE));
+				  MagickArraySize(image->columns,
+                                                  sizeof(JSAMPLE)));
   if (jpeg_pixels == (JSAMPLE *) NULL)
-    ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,image);
+    {
+      jpeg_destroy_decompress(&jpeg_info);
+      ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,image);
+    }
 
   /*
     Extended longjmp-based error handler (with jpeg_pixels)
@@ -1343,7 +1374,8 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
 	      *q++=image->colormap[index];
 	    }
 	}
-      else
+      else if ((jpeg_info.output_components == 3) ||
+               (jpeg_info.output_components == 4))
 	{
 	  if (jpeg_info.data_precision > 8)
 	    {
@@ -1762,16 +1794,19 @@ static void WriteXMPProfile(j_compress_ptr jpeg_info,
                             const size_t profile_length)
 {
   size_t
-    remaining,
     count,
     index,
-    header_length,
-    marker_length,
-    total_length;
+	header_length,
+	total_length;
+
+  unsigned int
+	marker_length,
+	remaining;
 
   header_length=strlen(xmp_std_header)+1; /* Include terminating null */
   total_length=header_length+profile_length;
-  remaining=total_length;
+  /* XMP profile must be no larger than range of 'unsigned int' */
+  remaining=(unsigned int) Min(UINT_MAX,total_length);
 
   marker_length=Min(remaining,JPEG_MARKER_MAX_SIZE);
   jpeg_write_m_header(jpeg_info,XML_MARKER,marker_length);
@@ -1879,8 +1914,12 @@ static void JPEGDestinationManager(j_compress_ptr cinfo,Image * image)
   destination->image=image;
 }
 
-static MagickPassFail WriteJPEGImage(const ImageInfo *image_info,Image *image)
+static MagickPassFail WriteJPEGImage(const ImageInfo *image_info,Image *imagep)
 {
+  Image
+    * volatile imagev = imagep,  /* volatile to avoid "clobber" */
+    *image;
+
   ErrorManager
     error_manager;
 
@@ -1936,16 +1975,33 @@ static MagickPassFail WriteJPEGImage(const ImageInfo *image_info,Image *image)
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
-  assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
-  status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
+  assert(imagep != (Image *) NULL);
+  assert(imagep->signature == MagickSignature);
+  status=OpenBlob(image_info,imagev,WriteBinaryBlobMode,&imagev->exception);
   if (status == False)
-    ThrowWriterException(FileOpenError,UnableToOpenFile,image);
+    ThrowWriterException(FileOpenError,UnableToOpenFile,imagev);
 
   (void) memset(&error_manager,0,sizeof(error_manager));
   (void) memset(&jpeg_info,0,sizeof(jpeg_info));
   (void) memset(&jpeg_error,0,sizeof(jpeg_error));
 
+  /*
+    Set initial longjmp based error handler.
+  */
+  jpeg_info.client_data=(void *) imagev;
+  jpeg_info.err=jpeg_std_error(&jpeg_error);
+  jpeg_info.err->emit_message=(void (*)(j_common_ptr,int)) JPEGMessageHandler;
+  jpeg_info.err->error_exit=(void (*)(j_common_ptr)) JPEGErrorHandler;
+  error_manager.image=imagev;
+  jpeg_info.client_data=(void *) &error_manager;
+  if (setjmp(error_manager.error_recovery))
+    {
+      jpeg_destroy_compress(&jpeg_info);
+      CloseBlob(imagev);
+      return MagickFail ;
+    }
+  image=imagev;  /* Use 'image' after this point for optimization */
+
   /*
     Transform image to user-requested colorspace.
   */
@@ -1975,22 +2031,6 @@ static MagickPassFail WriteJPEGImage(const ImageInfo *image_info,Image *image)
       return MagickFail;
     }
 
-  /*
-    Set initial longjmp based error handler.
-  */
-  jpeg_info.client_data=(void *) image;
-  jpeg_info.err=jpeg_std_error(&jpeg_error);
-  jpeg_info.err->emit_message=(void (*)(j_common_ptr,int)) JPEGMessageHandler;
-  jpeg_info.err->error_exit=(void (*)(j_common_ptr)) JPEGErrorHandler;
-  error_manager.image=image;
-  if (setjmp(error_manager.error_recovery))
-    {
-      jpeg_destroy_compress(&jpeg_info);
-      CloseBlob(image);
-      return MagickFail ;
-    }
-  jpeg_info.client_data=(void *) &error_manager;
-
   jpeg_create_compress(&jpeg_info);
   JPEGDestinationManager(&jpeg_info,image);
   jpeg_info.image_width=(unsigned int) image->columns;
