@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2003 - 2015 GraphicsMagick Group
+% Copyright (C) 2003 - 2016 GraphicsMagick Group
 % Copyright (C) 2002 ImageMagick Studio
 %
 % This program is covered by multiple licenses, which are described in
@@ -1293,7 +1293,8 @@ static unsigned int ReadConfigureFile(const char *basename,
     *xml;
 
   size_t
-    length;
+    length,
+    token_max_length;
 
   /*
     Read the delegates configure file.
@@ -1306,12 +1307,13 @@ static unsigned int ReadConfigureFile(const char *basename,
   if (xml == (char *) NULL)
     xml=AllocateString(DelegateMap);
   token=AllocateString(xml);
+  token_max_length=strlen(token);
   for (q=xml; *q != '\0'; )
     {
       /*
         Interpret XML.
       */
-      GetToken(q,&q,token);
+      MagickGetToken(q,&q,token,token_max_length);
       if (*token == '\0')
         break;
       (void) strlcpy(keyword,token,MaxTextExtent);
@@ -1321,7 +1323,7 @@ static unsigned int ReadConfigureFile(const char *basename,
             Comment element.
           */
           while ((LocaleNCompare(q,"->",2) != 0) && (*q != '\0'))
-            GetToken(q,&q,token);
+            MagickGetToken(q,&q,token,token_max_length);
           continue;
         }
       if (LocaleCompare(keyword,"<include") == 0)
@@ -1332,10 +1334,10 @@ static unsigned int ReadConfigureFile(const char *basename,
           while ((*token != '>') && (*q != '\0'))
             {
               (void) strlcpy(keyword,token,MaxTextExtent);
-              GetToken(q,&q,token);
+              MagickGetToken(q,&q,token,token_max_length);
               if (*token != '=')
                 continue;
-              GetToken(q,&q,token);
+              MagickGetToken(q,&q,token,token_max_length);
               if (LocaleCompare(keyword,"file") == 0)
                 {
                   if (depth > 200)
@@ -1385,11 +1387,11 @@ static unsigned int ReadConfigureFile(const char *basename,
         }
       if (delegate_list == (DelegateInfo *) NULL)
         continue;
-      GetToken(q,(char **) NULL,token);
+      MagickGetToken(q,(char **) NULL,token,token_max_length);
       if (*token != '=')
         continue;
-      GetToken(q,&q,token);
-      GetToken(q,&q,token);
+      MagickGetToken(q,&q,token,token_max_length);
+      MagickGetToken(q,&q,token,token_max_length);
       switch (*keyword)
         {
         case 'C':
